<?php

namespace Drupal\xb_ai_assistant\Plugin\AiAgent;

use Drupal\Core\Access\AccessResult;
use Drupal\Core\DependencyInjection\DependencySerializationTrait;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\ai_agents\Attribute\AiAgent;
use Drupal\ai_agents\PluginBase\AiAgentBase;
use Drupal\ai_agents\PluginInterfaces\AiAgentInterface;
use Drupal\Component\Serialization\Json;
use Drupal\xb_ai_assistant\XbPageBuilderHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the XB page builder Agent.
 */
#[AiAgent(
  id: 'xb_page_builder',
  label: new TranslatableMarkup('Xb Page Builder Agent'),
)]
class XbPageBuilder extends AiAgentBase implements ContainerFactoryPluginInterface {

  use DependencySerializationTrait;

  /**
   * The XbPageBuilderHelper.
   *
   * @var \Drupal\xb_ai_assistant\XbPageBuilderHelper
   */
  protected XbPageBuilderHelper $xbPageBuilderHelper;

  /**
   * {@inheritDoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    $parent_instance = parent::create($container, $configuration, $plugin_id, $plugin_definition);
    $parent_instance->xbPageBuilderHelper = $container->get('xb.page_builder_utility');
    return $parent_instance;
  }

  /**
   * Questions to ask.
   *
   * @var array
   */
  protected $questions;

  /**
   * The full data of the initial task.
   *
   * @var array
   */
  protected $data;

  /**
   * Task type.
   *
   * @var string
   */
  protected $taskType;


  /**
   * The entity.
   *
   * @var \Drupal\Core\Entity\EntityInterface
   */
  protected $entity;

  /**
   * {@inheritDoc}
   */
  public function getId() {
    return 'xb_page_builder';
  }

  /**
   * {@inheritDoc}
   */
  public function agentsNames() {
    return [
      'XB Page Builder',
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function agentsCapabilities() {
    return [
      'xb_page_builder' => [
        'name' => 'XB Page Builder agent',
        'description' => "This agent is capable of building page templates using experience builder(XB).",
        'usage_instructions' => "This agent is capable of building pages using Experience builder.",
        'inputs' => [
          'free_text' => [
            'name' => 'Prompt',
            'type' => 'string',
            'description' => 'The prompt to create a page using experience builder.',
            'default_value' => '',
          ],
        ],
        'outputs' => [],
      ],
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function setData($data) {
    $this->data[] = $data;
  }

  /**
   * {@inheritDoc}
   */
  public function isAvailable() {
    return TRUE;
  }

  /**
   * {@inheritDoc}
   */
  public function isNotAvailableMessage() {
    return $this->t('This is not available.');
  }

  /**
   * {@inheritDoc}
   */
  public function getRetries() {
    return 2;
  }

  /**
   * {@inheritDoc}
   */
  public function getData() {
    return $this->data;
  }

  /**
   * {@inheritDoc}
   */
  public function answerQuestion() {

    return parent::answerQuestion();
  }

  /**
   * {@inheritDoc}
   */
  public function hasAccess() {
    // Check for permissions.
    if (!$this->currentUser->hasPermission('access administration pages')) {
      return AccessResult::forbidden();
    }
    return parent::hasAccess();
  }

  /**
   * {@inheritDoc}
   */
  public function determineSolvability() {
    parent::determineSolvability();
    $this->taskType = $this->determineTypeOfTask();
    switch ($this->taskType) {
      case 'template_suggestion':
        return AiAgentInterface::JOB_SOLVABLE;
    }

    return AiAgentInterface::JOB_NOT_SOLVABLE;
  }

  /**
   * {@inheritDoc}
   */
  public function solve() {
    $message = '';
    foreach ($this->data as $data) {
      switch ($data['action']) {
        case 'template_suggestion':
          try {
            if ($data['status'] === 'success') {
              if ($data['component_info']) {
                $ai_suggested_layout = Json::decode($data['component_info'], TRUE);
                if ($ai_suggested_layout === NULL && json_last_error() !== JSON_ERROR_NONE) {
                  \Drupal::logger('xb_page_builder')->error($data['component_info']);
                  throw new \Exception('Failed to decode JSON: ' . json_last_error_msg());
                }
                // Apply the operations.
                $this->xbPageBuilderHelper->applyOperations($ai_suggested_layout['operations'], $this->entity);
                $message = 'The xb_page_builder agent task completed. Message from agent: ' . $ai_suggested_layout['message'];
              }
            }
            else {
              $message = 'The xb_page_builder agent task completed with errors. Message from agent: ' . Json::decode($data['component_info'])['message'];
            }
          }
          catch (\Exception $e) {
            $message = 'There was an error creating the node type: ' . $e->getMessage();
          }
          break;
      }
    }
    return $message;
  }

  /**
   * Check so all requirements are there.
   *
   * @return bool
   *   If all requirements are there.
   */
  public function checkRequirements() {
    return TRUE;
  }

  /**
   * Find the entity that needs to be updated.
   *
   * @return string
   *   The context.
   */
  public function determineTypeOfTask() {
    $task_description = $this->getTask()->getDescription();
    // Try to find the entity and component details from the task description.
    $parsed_task_description = $this->parseTaskDescription($task_description);

    if (!empty($parsed_task_description)) {
      $this->getTask()->setDescription($parsed_task_description['real_description']);
      $entity_type = $parsed_task_description['entity_type'];
      $entity_id = $parsed_task_description['entity_id'];
      $selected_component = $parsed_task_description['selected_component'] ?? 'None';
    }
    else {
      // Use the sub agent to find the entity and component details if parsing
      // the task description was unsuccessful.
      $data = $this->agentHelper->runSubAgent('findEntityAndComponentInfo', [
        'Context' => 'Entity and component details will be explicitly mentioned in the user prompt.',
      ]);
      $entity_type = $data[0]['entity_type'];
      $entity_id = $data[0]['entity_id'];
      $selected_component = $data[0]['selected_component'] ?? 'None';
    }

    if ($entity_type && $entity_id) {
      // Find the current layout.
      $entity = $this->entityTypeManager->getStorage($entity_type)->load($entity_id);
      $this->entity = $entity;
      $current_layout = $this->xbPageBuilderHelper->getCurrentLayout($entity);
      $available_components = $this->xbPageBuilderHelper->getValidSdcInfo();
      // Invoke the corresponding sub agents.
      if ($selected_component == 'None') {
        // If no component is selected, use the sub agent that can update
        // the entire layout.
        $data = $this->agentHelper->runSubAgent('createXbLayout', [
          'The current page layout is' => $current_layout,
          'The available components are' => $available_components,
        ]);
      }
      else {
        // If a component is selected, use the corresponding sub agent.
        $data = $this->agentHelper->runSubAgent('updateRelativeToExistingComponent', [
          'The current page layout is' => $current_layout,
          'The available components are' => $available_components,
          'The UUID of the selected component is' => $selected_component,
        ]);
      }

      if (isset($data[0]['action']) && $data[0]['action'] == 'template_suggestion' && $data[0]['component_info']) {
        $this->data = $data;
        return $data[0]['action'];
      }
    }
    throw new \Exception('Invalid action in Web Determining task.');
  }

  /**
   * Parse the task description and get the entity and component details.
   *
   * @param string $task_description
   *   The formatted task description string.
   */
  public function parseTaskDescription(string $task_description): array {
    $result = [];

    if (strpos($task_description, 'Page details: ') !== FALSE) {
      list($real_description, $page_details) = explode('Page details: ', $task_description, 2);
      $result['real_description'] = trim($real_description);

      $parts = explode(', ', $page_details);
      if (count($parts) === 3 && $result['real_description']) {
        foreach ($parts as $part) {
          list($key, $value) = explode(': ', $part);
          $result[$key] = $value;
        }

        if (isset($result['entity_type']) && isset($result['entity_id']) && isset($result['selected_component'])) {
          return $result;
        }
      }
    }

    return [];
  }

}
