<?php

namespace Drupal\xb_ai_assistant;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Site\Settings;
use Drupal\Core\Theme\ComponentPluginManager;
use Drupal\experience_builder\AutoSave\AutoSaveManager;
use Drupal\experience_builder\Controller\ApiLayoutController;
use Drupal\experience_builder\Entity\Component;
use Drupal\experience_builder\Plugin\ExperienceBuilder\ComponentSource\SingleDirectoryComponent;
use Symfony\Component\HttpFoundation\Request;

/**
 * Helper service for the XbPageBuilder agent.
 */
class XbPageBuilderHelper {

  /**
   * Constructor.
   */
  public function __construct(
    private readonly AutoSaveManager $autoSaveManager,
    private readonly ComponentPluginManager $componentPluginManager,
    private readonly EntityTypeManagerInterface $entityTypeManager,
    private readonly ApiLayoutController $apiLayoutController
  ) {
  }

  /**
   * Gets the data of all the usable SDC components.
   */
  public function getValidSdcInfo(): string {
    $sdc_definitions = $this->componentPluginManager->getDefinitions();
    /** @var \Drupal\experience_builder\Entity\Component[] $component_entities */
    $component_entities = $this->entityTypeManager->getStorage('component')->loadMultiple();
    $output = [];
    $ignore_xb_components = Settings::get('xb_ai_assistant_ignore_xb_sdc', FALSE);
    foreach ($sdc_definitions as $machine_name => $sdc_definition) {
      // Do not include the SDC provided by the xb module in the list
      // of available components.
      if ($ignore_xb_components && $sdc_definition['provider'] == 'experience_builder') {
        continue;
      }
      $component_id = SingleDirectoryComponent::convertMachineNameToId($machine_name);
      // Some sdc components would have invalid definitions or would be marked
      // as obsolete. Filter them.
      if (array_key_exists($component_id, $component_entities) && $component_entities[$component_id]->status()) {
        $output[$component_id] = [
          'label' => $sdc_definition['name'],
          'description' => $sdc_definition['description'] ?? 'No description available',
          'group' => $sdc_definition['group'] ?? '',
          'slots' => 'No Slots',
          'props' => 'No Props',
        ];

        // Get Slots.
        $slots = $sdc_definition['slots'] ?? [];
        if ($slots) {
          $output[$component_id]['slots'] = [];
          foreach ($slots as $slot => $details) {
            $output[$component_id]['slots'][$slot] = [
              'label' => $details['title'] ?? $slot,
              'description' => $details['description'] ?? 'No description available',
            ];
          }
        }

        // Get Props.
        $props = $sdc_definition['props']['properties'] ?? [];
        if ($props) {
          $output[$component_id]['props'] = [];
          foreach ($props as $prop_name => $prop_details) {
            $output[$component_id]['props'][$prop_name] = [
              'label' => $prop_details['title'] ?? $prop_name,
              'description' => $prop_details['description'] ?? 'No description available',
            ];
          }
        }
      }
    }

    return Json::encode($output);
  }

  private function generateComponentData(array $inputData): array {
    $data = [
      'source' => [],
      'resolved' => []
    ];

    // Check if 'propSources' exists in the input data.
    if (isset($inputData['propSources']) && is_array($inputData['propSources'])) {
      // Iterate over 'propSources' keys (text, style, element)
      foreach ($inputData['propSources'] as $key => $field) {
        // Only process fields with necessary subfields.
        if (isset($field['expression'], $field['sourceType'])) {
          $data['source'][$key] = [
            'expression' => $field['expression'],
            'sourceType' => $field['sourceType'],
          ];

          // Add default values.
          if (isset($field['default_values'])) {
            $data['source'][$key]['value'] = $field['default_values']['source'];
            $data['resolved'][$key] = $field['default_values']['resolved'] ?? $field['default_values']['source']['value'];
          }

          // Check for 'sourceTypeSettings' if it exists.
          if (isset($field['sourceTypeSettings'])) {
            $data['source'][$key]['sourceTypeSettings'] = $field['sourceTypeSettings'];
          }
        }
      }
    }

    return $data;
  }

  private function getComponentSlots(string $uuid, array $client_side_info): array {
    $slots = $client_side_info['metadata']['slots'] ?? [];
    $return = [];
    if (empty($slots)) {
      return $return;
    }
    else {
      foreach (array_keys($slots) as $slot_name) {
        $return[] = [
          'id' => $uuid . '/' . $slot_name,
          'name' => $slot_name,
          'nodeType' => 'slot',
          'components' => [],
        ];
      }
    }

    return $return;

  }

  public function getCurrentLayout(EntityInterface $entity): string {
    $autosave_data = $this->autoSaveManager->getAutoSaveData($entity)?->data ?? [];
    return Json::encode($this->formatAutosaveData($autosave_data));
  }

  private function formatAutosaveData(array $autosave_data): array {
    $components = [];
    $position = 0;
    if ($autosave_data) {
      foreach ($autosave_data['layout'][0]['components'] as $component) {
        $this->extractComponentData($component, 'content', $components, $position);
      }
    }
    return ['components' => $components];
  }

  private function extractComponentData(array $component, string $parentId, array &$components, int &$position): void {
    $components[] = [
      'uuid' => $component['uuid'],
      'name' => $component['type'],
      'parentId' => $parentId,
      'position' => $position++,
    ];

    if (!empty($component['slots'])) {
      foreach ($component['slots'] as $slot) {
        $slot_position = 0;
        foreach ($slot['components'] as $slot_component) {
          $this->extractComponentData($slot_component, $slot['id'], $components, $slot_position);
        }
      }
    }
  }

  public function getLayoutModelStructureOfComponent(Component $component, string $uuid): array {
    $structure = [];
    $component_source = $component->getComponentSource();
    $client_side_info = $component_source->getClientSideInfo($component);
    $component_data = $this->generateComponentData($client_side_info);
    $component_data['name'] = $component->label();
    $structure['layout'] = [
      'slots' => $this->getComponentSlots($uuid, $client_side_info),
      'nodeType' => 'component',
      'type' => $component->id(),
      'uuid' => $uuid,
    ];
    $structure['model'][$uuid] = $component_data;

    return $structure;
  }

  public function applyOperations(array $operations, EntityInterface $entity): void {
    $autosave_data = $this->autoSaveManager->getAutoSaveData($entity)?->data;
    if (!$autosave_data) {
      $autosave_data = $this->apiLayoutController->get($entity)->additionalData;
      // If no components are present, $autosave_data['model'] will be an object.
      if (isset($autosave_data['model']) && is_object($autosave_data['model'])) {
        $autosave_data['model'] = [];
      }
    }
    foreach ($operations as $operation) {
      switch ($operation['operation']) {
        case 'ADD':
          $this->addComponents($operation['components'], $autosave_data);
          break;

        case 'MOVE':
          $this->moveComponents($operation['components'], $autosave_data);
          break;
      }
    }

    // The rendered HTML of the page should also be saved in tempstore.
    // See https://www.drupal.org/project/experience_builder/issues/3492061.
    // Since the method \Drupal\experience_builder\Controller\ApiLayoutController::buildPreviewRenderable
    // is private and cannot be used here, relying on
    // \Drupal\experience_builder\Controller\ApiLayoutController::post, which
    // generates the HTML and updates the autosave data.
    $request_obj = new Request([], [], [], [], [], [], Json::encode($autosave_data));
    $this->apiLayoutController->post($request_obj, $entity);
  }

  private function addComponents(array $components, array &$autosave_data): array {
    foreach ($components as $component) {
      $this->validateComponent($component);
      $component_entity = Component::load($component['name']);
      if (!$component_entity) {
        throw new \Exception(sprintf('Component not found: %s', $component['name']));
      }
      $component_data = $this->getLayoutModelStructureOfComponent($component_entity, $component['uuid']);

      if ($component['parentId'] === 'content') {
        array_splice($autosave_data['layout'][0]['components'], $component['position'], 0, [$component_data['layout']]);
      }
      else {
        $this->addComponentToSlot($autosave_data['layout'][0]['components'], $component, $component_data);
      }
      $autosave_data['model'] += $component_data['model'];
    }

    return $autosave_data;
  }

  private function moveComponents(array $components, array &$autosave_data): array {
    foreach ($components as $component) {
      $this->validateComponent($component);
      $uuid = $component['uuid'];
      assert($component['prevParentId']);
      $prev_parent_id = $component['prevParentId'];
      $parent_id = $component['parentId'];
      $position = $component['position'];

      // Find and remove the component from its previous location.
      $removed_component = NULL;
      if ($prev_parent_id === 'content') {
        foreach ($autosave_data['layout'][0]['components'] as $index => $existing_component) {
          if ($existing_component['uuid'] === $uuid) {
            $removed_component = array_splice($autosave_data['layout'][0]['components'], $index, 1)[0];
            break;
          }
        }
      }
      else {
        $this->removeComponentFromSlot($autosave_data['layout'][0]['components'], $uuid, $prev_parent_id, $removed_component);
      }

      if ($removed_component === NULL) {
        throw new \Exception(sprintf('Component not found: %s', $uuid));
      }

      // Add the component to its new position.
      if ($parent_id === 'content') {
        array_splice($autosave_data['layout'][0]['components'], $position, 0, [$removed_component]);
      }
      else {
        $this->addComponentToSlot($autosave_data['layout'][0]['components'], $component, ['layout' => $removed_component]);
      }
    }

    return $autosave_data;
  }

  private function removeComponentFromSlot(array &$components, string $uuid, string $slot_id, &$removed_component) {
    foreach ($components as &$existing_component) {
      if (!empty($existing_component['slots'])) {
        foreach ($existing_component['slots'] as &$slot) {
          if ($slot['id'] === $slot_id) {
            foreach ($slot['components'] as $index => $slot_component) {
              if ($slot_component['uuid'] === $uuid) {
                $removed_component = array_splice($slot['components'], $index, 1)[0];
                return;
              }
            }
          } else {
            // Check nested slots.
            if (!empty($slot['components'])) {
              $this->removeComponentFromSlot($slot['components'], $uuid, $slot_id, $removed_component);
            }
          }
        }
      }
    }
  }

  private function validateComponent(array $component) {
    $required_keys = ['uuid', 'position', 'parentId', 'name'];
    foreach ($required_keys as $key) {
      if (!isset($component[$key])) {
        throw new \Exception(sprintf('Missing required component key: %s', $key));
      }
    }
  }

  private function addComponentToSlot(array &$components, array $component, array $component_data) {
    foreach ($components as &$existing_component) {
      if (!empty($existing_component['slots'])) {
        foreach ($existing_component['slots'] as &$slot) {
          if ($slot['id'] === $component['parentId']) {
            array_splice($slot['components'], $component['position'], 0, [$component_data['layout']]);
            return;
          }
          else {
            // Check nested slots.
            if (!empty($slot['components'])) {
              $this->addComponentToSlot($slot['components'], $component, $component_data);
            }
          }
        }
      }
    }
  }

}
