import { useSelector, useDispatch } from 'react-redux';
import { useState } from 'react';
import Button from './Button';

// Constants
const API_ENDPOINTS = {
  ASSISTANT: '/xb-assistant',
  LAYOUT: '/xb/api/v0/layout'
};

const XbAiAssistantExtension = () => {
  const dispatch = useDispatch();
  const [userInput, setUserInput] = useState('');
  const [response, setResponse] = useState(null);
  const [isLoading, setIsLoading] = useState(false);

  // Get the current layout and available components from the Redux store.
  const selectedComponent = useSelector(state => state.ui.selection.items[0]);
  const theLayout = useSelector(state => state?.layoutModel?.present?.layout);
  const availableComponents = useSelector(
    state => state?.componentAndLayoutApi?.queries['getComponents(undefined)']?.data
  );

  // Entity information from Drupal settings
  const { entity: entityId, entityType, componentSelectionUtils } = drupalSettings.xb || {};

  /**
   * Recursively search for the 'type' of the selected component.
   */
  const findComponentType = (components, targetUuid) => {
    if (!components || !Array.isArray(components)) return null;
    for (const component of components) {
      if (component.uuid === targetUuid) {
        return component.type;
      }

      if (component.slots) {
        for (const slot of component.slots) {
          const type = findComponentType(slot.components, targetUuid);
          if (type) return type;
        }
      }
    }
    return null;
  }

  /**
   * Get the selected component's display name
   */
  const getSelectedComponentName = () => {
    if (!selectedComponent || !Array.isArray(theLayout) || !availableComponents) {
      return null;
    }

    // Find the component type from the layout
    let selectedType = null;
    for (const region of theLayout) {
      const type = findComponentType(region.components, selectedComponent);
      if (type) {
        selectedType = type;
        break;
      }
    }

    // Find the component name from available components using the 'type' value.
    if (
      selectedType &&
      availableComponents &&
      typeof availableComponents === 'object' &&
      availableComponents[selectedType]
    ) {
      return availableComponents[selectedType].name || null;
    }

    return null;
  };

  const selectedComponentName = getSelectedComponentName();
  if (selectedComponentName == null) {componentSelectionUtils.unsetSelectedComponent();};

  /**
   * Handle form submission
   */
  const handleSubmit = async (e) => {
    e.preventDefault();

    if (!userInput.trim()) return;

    try {
      setIsLoading(true);
      setResponse(null);

      // Process request to assistant API
      const assistantResponse = await fetchAssistantResponse();

      if (assistantResponse) {
        // Refresh the layout to show the changes
        await refreshLayout();
      }
    } catch (error) {
      console.error('Error processing request:', error);
    } finally {
      setIsLoading(false);
      setUserInput('');
    }
  };

  /**
   * Fetch assistant response
   */
  const fetchAssistantResponse = async () => {
    const response = await fetch(`${window.location.origin}${API_ENDPOINTS.ASSISTANT}`, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json',
      },
      body: JSON.stringify({
        prompt: userInput,
        entityType,
        entityId,
        selectedComponent
      }),
    });

    if (!response.ok) {
      console.error('Failed to get AI assistant response:', response.statusText);
      return null;
    }

    const data = await response.json();
    setResponse(data);
    return data;
  };

  /**
   * Refresh layout after changes
   */
  const refreshLayout = async () => {
    const apiResponse = await fetch(
      `${window.location.origin}${API_ENDPOINTS.LAYOUT}/${entityType}/${entityId}`
    );

    if (!apiResponse.ok) {
      console.error('Failed to refresh the layout:', apiResponse.statusText);
      return;
    }

    const apiData = await apiResponse.json();
    dispatch({
      type: 'layoutModel/setLayoutModel',
      payload: {
        layout: apiData.layout,
        model: apiData.model,
        updatePreview: true,
      }
    });
  };

  /**
   * Handler for deselecting component
   */
  const handleDeselect = () => {
    componentSelectionUtils?.unsetSelectedComponent();
  };

  return (
    <div className="xb-ai-assistant-extension">
      <div className="component-selection" style={{ marginBottom: '2em' }}>
        {selectedComponentName ? (
          <div className="selected-component-container">
            <div className="selected-component" style={{ display: 'flex', alignItems: 'center' }}>
              <p style={{ margin: '0px' }}>
                Component selected: <strong>{selectedComponentName}</strong>
              </p>
              <Button
                data-testid="ex-log-button"
                onClick={handleDeselect}
              >
                Deselect
              </Button>
            </div>
            <p className="selection-notice" style={{ marginTop: '0px', fontSize: 'small' }}>Changes will affect this component.</p>
          </div>
        ) : (
          <p className="no-selection-notice">
            No component selected. Updates will be made to the entire layout.
          </p>
        )}
      </div>

      <form onSubmit={handleSubmit}>
        <textarea
          data-testid="ex-textarea"
          value={userInput}
          onChange={(e) => setUserInput(e.target.value)}
          placeholder="Type something here..."
          required
          style={{ width: '100%', height: '100px' }}
          disabled={isLoading}
        />
        <Button
          data-testid="ex-log-button"
          type="submit"
          disabled={isLoading}
        >
          {isLoading ? 'Processing...' : 'Generate'}
        </Button>
      </form>

      {response && (
        <div style={{ marginTop: '10px', fontWeight: 'bold' }}>
          {response.message}
        </div>
      )}
    </div>
  );
};

export default XbAiAssistantExtension;
